#include <p18f4550.h>
#include <i2cEEPROM.h>
#include <delay.h>
#include <intEEPROM.h>
//#include <i2cEEPROM.c>
//#include <delay.c>
//#include <i2c.h>

#pragma config FOSC = HS
#pragma config WDT = OFF
#pragma config LVP = OFF
#pragma config PBADEN = OFF

//OSC = HS Impostato per lavorare ad alta frequenza
//WDT = OFF Disabilito il watchdog timer
//LVP = OFF Disabilito programmazione LVP
//PBADEN = OFF Disabilito gli ingressi analogici

//************************************************************
// Dichiarazione Prototipi di funzione
//************************************************************

void Low_Int_Event (void);								// Gestione interrupt bassa priorit
void High_Int_Event (void);								// Gestione interrupt alta priorit
void Impostazioni_porte (void);							// Impostazioni porte microcontrollore
void Impostazioni_interrupt_alta (void);				// Impostazioni interrupt alta priorit
void Impostazioni_interrupt_bassa (void);				// Impostazioni interrupt bassa priorit
void Attiva_interrupt (void);							// Abilita gli interrupt
void Imposta_ADC (void);								// Imposta e abilita l'ADC
int Valore_ADC (char canale);							// Legge un canale dell'ADC
void Scrivi_EEPROM_int (int dato, unsigned int ind);	// Scrive nella EEPROM interna al PIC
int Leggi_EEPROM_int (unsigned int ind);				// Legge nella EEPROM interna al PIC
void Inizializza_I2C (void);							// Inizializza il modulo I2C
void Scrivi_EEPROM_est (int dato, unsigned int ind);	// Scrive nella EEPROM esterna al PIC
unsigned char Leggi_EEPROM_est (unsigned int ind);		// Legge nella EEPROM esterna al PIC


//************************************************************
// Impostazione e Gestione priorit alta
//************************************************************

#pragma code high_vector = 0x08

void high_interrupt (void) {
  // Salto per la gestione dell'interrupt ad alta priorita`
  _asm GOTO High_Int_Event _endasm
}

#pragma code
#pragma interrupt High_Int_Event

//************************************************************
// Impostazione e Gestione priorit bassa
//************************************************************

#pragma code low_vector = 0x18

void low_interrupt (void) {
  // Salto per la gestione dell'interrupt a bassa priorita`
  _asm GOTO Low_Int_Event _endasm
}

#pragma code
#pragma interruptlow Low_Int_Event

//************************************************************
// PROGRAMMA PRINCIPALE
//************************************************************

void main (void){

unsigned int address = 1;
char canale = 0;
int lettura;

Impostazioni_porte();
Impostazioni_interrupt_alta();
Impostazioni_interrupt_bassa();
Attiva_interrupt();
Imposta_ADC();
Inizializza_I2C();

while (1) {

  lettura = Valore_ADC(canale);
  lettura = lettura >> 6;
  
  Scrivi_EEPROM_int(lettura, address);
  LATD = Leggi_EEPROM_int(address);

  Scrivi_EEPROM_est (lettura, address);
  LATD = Leggi_EEPROM_est (address);

}
}

//************************************************************
// Funzione per la gestione dell'interrupt ad alta priorit
//************************************************************

void High_Int_Event (void) {
  // Controllo che l'interrupt sia stato generato da INT2
  if (INTCON3bits.INT2IF == 1 ) {
    // Inverto lo stato del LED 2
    LATBbits.LATB3 = ~LATBbits.LATB3;
    //pausa filtraggio spike
    delay_ms(200);
    // Resetto il flag d'interrupt per permettere nuove interruzioni
    INTCON3bits.INT2IF = 0;
  }
}

//************************************************************
// Funzione per la gestione dell'interrupt a bassa priorit
//************************************************************

void Low_Int_Event (void) {
  // Controllo che l'interrupt sia stato generato da Timer0
  if (INTCONbits.TMR0IF == 1 ) {
      // Non eseguo il filtraggio spike perch l'interrupt  stato generato dal Timer0
      // Inverto lo stato del LED 1
      LATBbits.LATB5 = ~LATBbits.LATB5;
  // Resetto il flag d'interrupt per permettere nuove interruzioni
  INTCONbits.TMR0IF = 0;
  }
  if (PIR2bits.TMR3IF == 1 ) {
    PIR2bits.TMR3IF = 0;
    T3CONbits.TMR3ON = 0;
  }
}


//************************************************************
// Imposto PORTA-B-C-D-E input/output
//************************************************************

void Impostazioni_porte (void) {

  // Imposto il quarzo per la libreria delay
  setQuartz (20);

  // Imposto PORTA tutti ingressi
  LATA = 0x00; // pongo a zero le uscite
  TRISA = 0xFF;
 
  // Imposto PORTB
  LATB = 0x00;
  TRISB = 0b11010111;

  // Imposto PORTC tutti ingressi eccetto RC2 output
  LATC = 0x00;
  TRISC = 0b11111011;

  // Imposto PORTD
  LATD = 0x00;
  TRISD = 0b11110000;

  // Imposto PORTE tutti ingressi
  LATE = 0x00;
  TRISE = 0xFF;

  return;
}

//************************************************************
// Impostazioni degli interrupt ad alta priorit
//************************************************************

void Impostazioni_interrupt_alta (void) {

  // Abilita i resistori di pull-up sulla PORTB
  EnablePullups();

  // Definisco l'interrupt su INT2 sul rising edge
  INTCON2bits.INTEDG2 = 1;

  // Abilito le interruzioni su INT2
  INTCON3bits.INT2IE = 1;

  // Definisco l'interrupt su INT2 come ad alta priorit
  INTCON3bits.INT2IP = 1;

  return;
}

//************************************************************
// Impostazioni degli interrupt a bassa priorit
//************************************************************

void Impostazioni_interrupt_bassa (void) {

  // Timer0 in modalit 16 bit
  T0CONbits.T08BIT = 0;
  // Timer0 clock interno
  T0CONbits.T0CS = 0;
  // Timer0 Prescaler abilitato
  T0CONbits.PSA = 0;
  // Timer0 Prescaler 32
  T0CONbits.T0PS0 = 0;
  T0CONbits.T0PS1 = 0;
  T0CONbits.T0PS2 = 1;
  // Abilito le interruzioni del Timer0
  INTCONbits.TMR0IE = 1;
  // Abilito le interruzioni del Timer0 come bassa priorit
  INTCON2bits.TMR0IP = 0;
  // Abilito il Timer0
  T0CONbits.TMR0ON = 1;

  // Timer3 in modalit 16 bit, Prescaler 1:1, Timer1 per i moduli CCP
  // In questo modo il Timer3 genera un overflow ogni 15 ms circa
  T3CON = 0;
  // Timer3 clock interno
  T3CONbits.TMR3CS = 0;
  // Abilito le interruzioni del Timer0
  PIE2bits.TMR3IE = 1;
  // Abilito le interruzioni del Timer0 come bassa priorit
  IPR2bits.TMR3IP = 0;
  // Disabilito il Timer3
  T3CONbits.TMR3ON = 0;

  return;
}

//************************************************************
// Abilito gli interrupt
//************************************************************

void Attiva_interrupt (void) {

  // Abilito modalita` interruzione a due livelli alta e bassa
  RCONbits.IPEN = 1;
  // Abilito gli interrupt ad alta priorita`
  INTCONbits.GIEH = 1;
  // Abilito gli interrupt a bassa priorita`
  INTCONbits.GIEL = 1 ;

  return;
}

//************************************************************
// Imposto ed abilito l'ADC
//************************************************************

void Imposta_ADC (void) {

// Abilito RA0-RA3 come ingressi analogico. VREF sono impostate a massa e VCC
ADCON1 = 0b00001011;

// Imposto i tempi di conversione e giustificazione a destra
// TAD : FOSC/4
// TACQ: 16 TAD
ADCON2 = 0b10110100;

  return;
}

//************************************************************
// Funzione per la lettura di un canale dell'ADC
//************************************************************

int Valore_ADC (char canale) {

  int lettura;

  // Seleziono il canale d'ingresso e abilito l'ADC
  //ADCON0 = (canale << 2) + 1;
  ADCON0 = 0b00000001;

  // Avvio la conversione Analogico/Digitale
  ADCON0bits.GO = 1;

  // Attendo la fine della conversione
  while(ADCON0bits.GO);

  // Prelevo il valore della conversione
  lettura = (((int) ADRESH) << 8) | ADRESL; // valore a 10 bits
  return(lettura);
}

//************************************************************
// Funzione per la scrittura nella EEPROM interna al PIC
//************************************************************

void Scrivi_EEPROM_int (int dato, unsigned int ind) {

  // Scrivo il dato all'indirizzo della EEPROM specificato
  write_Int_EEPROM (dato, ind);

  // Azzero il Timer3 per riprendere il conto alla rovescia
  TMR3H = 0;
  TMR3L = 0;

  // Abilito il Timer3
  T3CONbits.TMR3ON = 1;

  return;
}

//************************************************************
// Funzione per la lettura nella EEPROM interna al PIC
//************************************************************

int Leggi_EEPROM_int (unsigned int ind) {

  // Attendo che siano passati 15 ms circa dall'ultima scrittura
  // Attendo che Timer3 vada in overflow facendo scattare l'interrupt che lo disattiva
  while(T3CONbits.TMR3ON);

  // Leggo il dato dalla EEPROM interna
  return(read_Int_EEPROM (ind));
}

//************************************************************
// Inizializzo il modulo I2C
//************************************************************

void Inizializza_I2C (void) {

  // Inizializzo il modulo I2C a 400KHz @20MHz
  OpenI2C(MASTER, SLEW_ON);
  SSPADD = 12;

}

//************************************************************
// Funzione per la scrittura nella EEPROM esterna al PIC
//************************************************************

void Scrivi_EEPROM_est (int dato, unsigned int ind) {

  // Scrivo il dato all'indirizzo della EEPROM specificato
  write_I2C_EEPROM (0xA0, ind, dato);

  // Azzero il Timer3 per riprendere il conto alla rovescia
  TMR3H = 0;
  TMR3L = 0;

  // Abilito il Timer3
  T3CONbits.TMR3ON = 1;

  return;
}

//************************************************************
// Funzione per la lettura nella EEPROM esterna al PIC
//************************************************************

unsigned char Leggi_EEPROM_est (unsigned int ind) {

  // Variabile per salvare il dato di ritorno
  unsigned char data = 0;

  // Attendo che siano passati 15 ms circa dall'ultima scrittura
  // Attendo che Timer3 vada in overflow facendo scattare l'interrupt che lo disattiva
  while(T3CONbits.TMR3ON);

  // Leggo il dato dalla EEPROM interna
  read_I2C_EEPROM (0xA0, ind, &data);
  return(data);
}
